#!/bin/bash
###########################################################################
# Eole NG - 2007  
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr 
#  
# restauration.sh
#  
# script de restauration d'une archive de sauvegarde zephir NG
#       
###########################################################################

# color ASCII codes
#
DEFAULT="\e[0m"
BOLD="\e[1m"
BLACK="\e[30m"
RED="\e[31m"
GREEN="\e[32m"
YELLOW="\e[33m"
BLUE="\e[34m"
VIOLET="\e[35m"
CYAN="\e[36m"
WHITE="\e[37m"

Lock="sauvegarde"
Excl="maj"

RepEole=${RepEole="/usr/share/eole"}
. ParseDico

backup_dir=/var/lib/zephir_backups
pg_dir=/var/lib/postgres
pg_perms=$pg_dir/pg_hba.conf
pg_service=postgresql-8.2
initdb=/usr/lib/postgresql/8.2/bin/initdb

if [ ! -d $backup_dir ]
then
	echo -e "${RED} Erreur, répertoire des sauvegardes non trouvé ${DEFAULT}"
	exit 1
fi
pushd $backup_dir &>/dev/null 2>&1

if [ ! -z $1 ]
then
	archive=$1
        basename=`echo -e $archive | sed -e s/"\.tar\.gz"/""/`
else
	archive=/tmp
fi

while [ ! -f $archive ]
do
	# pas de fichier passé en paramètre
	save_files=`ls *.tar.gz | sed -e s/".tar.gz"/""/`
	echo -e "     Utilitaire de restauration zephir"
	echo -e
	echo -e "${RED}!!Attention : toutes les modifications effectuées"
	echo -e "  après la sauvegarde restaurée seront perdues!!${DEFAULT}"
	echo -e
	echo -e "${CYAN}liste des sauvegardes présentes :${DEFAULT}"
	echo -e
	echo -e "$save_files"
	echo -e
	read -p "sauvegarde à restaurer (rien pour sortir): " basename
	if [ -z $basename ]
	then
		exit 0
	fi
	if [ -f $basename.tar.gz ]
	then
		archive=$basename.tar.gz
	else
		echo -e "fichier invalide : $basename.tar.gz"
	fi
done

echo -e "arrêt du service zephir..."
service zephir stop > /dev/null 2>&1
echo -e "décompression en cours..."
tar xzf $archive
cd $basename
echo -e "vérification des données..."
if [ ! -f backup_db.sql ]
then
	echo -e "${RED}sauvegarde de la base de données non trouvée${DEFAULT}"
	exit 1
fi
if [ ! -f ldap.ldif ]
then
	echo -e "${RED}sauvegarde de la base ldap non trouvée${DEFAULT}"
	exit 1
fi
if [ ! -f var_lib_zephir.tar ]
then
	echo -e "${RED}sauvegarde des configurations des clients non trouvée${DEFAULT}"
	exit 1
fi
if [ ! -f etc_uucp.tar ]
then
	echo -e "${RED}sauvegarde des configurations uucp non trouvée${DEFAULT}"
	exit 1
fi
if [ ! -f var_spool_uucp.tar ]
then
	echo -e "${RED}sauvegarde des clés de connexion des clients non trouvée${DEFAULT}"
	exit 1
fi
if [ ! -f etc_eole.tar ]
then
	echo -e "${RED}sauvegarde de la configuration du serveur non trouvée${DEFAULT}"
	exit 1
fi

rep="N"
# restauration des données de la base postgresql
read -p "Restaurer la base de données (o/n) ?" rep
echo -e
if [ "$rep" == "o" ] || [ "$rep" == "O" ]
then
	echo -e " - base postgresql"
	cp -f $pg_perms /tmp/pg_hba.savezephir
	cp -f $pg_dir/postgresql.conf /tmp/postgresql.conf
	service $pg_service stop &>/dev/null
       	rm -rf $pg_dir/pgdata/*
	echo -e "   initialisation de la base"
	su - postgres -c "$initdb -D $pg_dir/pgdata/" &> /dev/null
	# postgres : se mettre en mode sans mot de passe
	echo -e -e "local\tall\tall\ttrust" > $pg_perms
	chown postgres.postgres $pg_perms
	chmod 600 $pg_perms
	mv -f /tmp/postgresql.conf $pg_dir/postgresql.conf
	chown postgres.postgres $pg_dir/postgresql.conf
	chmod 600 $pg_dir/postgresql.conf
	service $pg_service start &> /dev/null
	sleep 2
	echo -e "   injection des données"
	su postgres -c "psql template1 -f backup_db.sql" > /dev/null 2>&1
	# recréation du mot de passe
	echo -e "   regénération du mot de passe"
	/usr/share/eole/gen_pwd.py &> /dev/null
	echo -e "   mise à jour du schéma de la base"
	su postgres -c "psql -d zephir -U zephir -f /usr/share/zephir/sql/maj.sql" > /dev/null 2>&1
	# postgres : se remettre en mode mot de passe
	rm -f $pg_perms
	mv -f /tmp/pg_hba.savezephir $pg_perms
	chown postgres.postgres $pg_perms
	chmod 600 $pg_perms
	service $pg_service restart &>/dev/null
fi

# restauration des répertoires et fichiers importants
# pushd /var/lib > /dev/null 2>&1

echo -e " - base ldap"
service slapd stop >/dev/null 2>&1
rm -rf /var/lib/ldap/*
# restauration de la base ldap
# tar -C /var/lib/ -xpf var_lib_ldap.tar >/dev/null 2>&1
slapadd -l ldap.ldif >/dev/null 2>&1
chown -R openldap.openldap /var/lib/ldap
service slapd start >/dev/null 2>&1
echo -e " - configuration des serveurs"
# /var/lib/zephir/
rm -rf /var/lib/zephir
tar -C /var/lib/ -xpf var_lib_zephir.tar >/dev/null 2>&1
# /etc/uucp/
rm -rf /etc/uucp
tar -C /etc/ -xpf etc_uucp.tar >/dev/null 2>&1
# /var/spool/uucp/
rm -rf /var/spool/uucp
tar -C /var/spool -xpf var_spool_uucp.tar >/dev/null 2>&1
echo -e " - configuration eole"
rm -rf /etc/eole/dicos /etc/eole/patch /etc/eole/config.eol
tar -C /etc/eole -xpf etc_eole.tar >/dev/null 2>&1
# vérification du dictionnaire
echo -e
if [ -f /root/zephir.eol ]
then
	diff -q /root/zephir.eol /etc/eole/config.eol &>/dev/null
	if [ ! $? -eq 0 ]
	then
		mv /root/zephir.eol /root/zephir.eol.bak
		echo -e "${CYAN}Le fichier /root/zephir.eol est différent de la version restaurée"
		echo -e "Le fichier présent actuellement a été renommé en /root/zephir.eol.bak"
		echo -e
		echo -e "Réinstanciez le serveur avec l'un des 2 fichiers sans écraser la base de données${DEFAULT}"
		echo -e
	else
		echo -e "${CYAN}"
		echo -e "Reconfigurez le serveur après la fin de la restauration"
		echo -e "${DEFAULT}"
	fi
fi
cp -f /etc/eole/config.eol /root/zephir.eol
 
# création du fichier de sauvegarde compressé
echo -e "Supression du répertoire temporaire..."
rm -rf $backup_dir/$basename
echo -e "${GREEN}démarrage du service zephir"
service zephir start > /dev/null 2>&1
echo -e "Systeme restauré${DEFAULT}"
echo -e
# retour au répertoire d'origine
popd >/dev/null 2>&1
